@@ -1,13 +1,11 @@
 # Locale::Po4a::Pod -- Convert POD data to PO file, for translation.
-# $Id: Chooser.pm,v 1.18 2004/08/17 23:40:05 mquinson Exp $
-#
-# Copyright 2002 by Martin Quinson <Martin.Quinson@ens-lyon.fr>
+# $Id: Chooser.pm,v 1.25 2005/02/27 23:32:02 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
 #
 # This module converts POD to PO file, so that it becomes possible to 
-# translate POD formated documentation. See gettext documentation for
+# translate POD formatted documentation. See gettext documentation for
 # more info about PO files.
 
 ############################################################################
@@ -21,25 +19,32 @@ use 5.006;
 use strict;
 use warnings;
 use Locale::gettext;
+use Locale::Po4a::Common;
 
 sub new {
     my ($module)=shift;
     my (%options)=@_;
 
-    die gettext("Need to provide a module name to the Chooser")."\n"
+    die wrap_mod("po4a::chooser", gettext("Need to provide a module name"))
       unless defined $module;
 
     my $modname;
     if ($module eq 'kernelhelp') {
         $modname = 'KernelHelp';
+    } elsif ($module eq 'newsdebian') {
+        $modname = 'NewsDebian';
+    } elsif ($module eq 'latex') {
+        $modname = 'LaTeX';
     } else {
         $modname = ucfirst($module);
     }
     if (! UNIVERSAL::can("Locale::Po4a::$modname", 'new')) {
         eval qq{use Locale::Po4a::$modname};
         if ($@) {
-            warn sprintf(gettext("Unknown format type: %s."), $module)."\n";
-	    warn sprintf(gettext("Module loading error: %s"), $@)."\n" if $options{'verbose'} > 0;
+            warn wrap_msg(gettext("Unknown format type: %s."), $module);
+	    warn wrap_mod("po4a::chooser",
+		gettext("Module loading error: %s"), $@)
+	      if defined $options{'verbose'} && $options{'verbose'} > 0;
             list(1);
         }
     }
@@ -47,14 +52,17 @@ sub new {
 }
 
 sub list {
-    warn gettext("List of valid formats:\n".
-		 "  - kernelhelp: Help messages of each kernel compilation option.\n".
-#		 "  - html: HTML documents (EXPERIMENTAL).\n".
-		 "  - man: Good old manual page format.\n".
-		 "  - pod: Perl Online Documentation format.\n".
-		 "  - sgml: either debiandoc or docbook DTD.\n".
-		 "  - dia: uncompressed Dia diagrams.\n".
-		 "  - guide: Gentoo Linux's xml documentation format.")."\n";
+    warn wrap_msg(gettext("List of valid formats:")
+	."\n  - ".gettext("kernelhelp: Help messages of each kernel compilation option.")
+#	."\n  - ".gettext("html: HTML documents (EXPERIMENTAL).")
+	."\n  - ".gettext("man: Good old manual page format.")
+	."\n  - ".gettext("pod: Perl Online Documentation format.")
+	."\n  - ".gettext("sgml: either debiandoc or docbook DTD.")
+	."\n  - ".gettext("dia: uncompressed Dia diagrams.")
+	."\n  - ".gettext("guide: Gentoo Linux's xml documentation format.")
+	."\n  - ".gettext("docbook: Docbook XML.")
+	."\n  - ".gettext("latex: LaTeX format.")
+    );
     exit shift;
 }
 ##############################################################################
@@ -72,7 +80,7 @@ Locale::Po4a::Chooser - Manage po4a modules
 
 Locale::Po4a::Chooser is a module to manage po4a modules. Before, all po4a
 binaries used to know all po4a modules (pod, man, sgml, etc). This made the
-add of a new module a boring, to make sure the documentation is syncronized
+add of a new module boring, to make sure the documentation is synchronized
 in all modules, and that each of them can access the new module.
 
 Now, you just have to call the Locale::Po4a::Chooser::new() function,
@@ -98,20 +106,21 @@ L<Locale::Po4a::Man(3pm)>,
 L<Locale::Po4a::Pod(3pm)>,
 L<Locale::Po4a::Sgml(3pm)>,
 L<Locale::Po4a::Html(3pm)>,
-L<Locale::Po4a::Dia(3pm)>.
+L<Locale::Po4a::Dia(3pm)>,
+L<Locale::Po4a::LateX(3pm)>.
 
 =back
 
 =head1 AUTHORS
 
  Denis Barbier <barbier@linuxfr.org>
- Martin Quinson <martin.quinson@tuxfamily.org>
+ Martin Quinson (mquinson#debian.org)
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2002 by SPI, inc.
+Copyright 2002,2003,2004,2005 by SPI, inc.
 
 This program is free software; you may redistribute it and/or modify it
-under the terms of GPL (see COPYING file).
+under the terms of GPL (see the COPYING file).
 
 =cut
