@@ -1,13 +1,11 @@
 # Locale::Po4a::Pod -- Convert POD data to PO file, for translation.
-# $Id: KernelHelp.pm,v 1.8 2004/08/27 18:18:34 barbier Exp $
-#
-# Copyright 2002 by Martin Quinson <Martin.Quinson@ens-lyon.fr>
+# $Id: KernelHelp.pm,v 1.12 2005/02/27 22:53:52 mquinson Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
 #
 # This module converts POD to PO file, so that it becomes possible to 
-# translate POD formated documentation. See gettext documentation for
+# translate POD formatted documentation. See gettext documentation for
 # more info about PO files.
 
 ############################################################################
@@ -16,6 +14,7 @@
 
 use Pod::Parser;
 use Locale::Po4a::TransTractor qw(process new);
+use Locale::Po4a::Common;
 
 package Locale::Po4a::KernelHelp;
 
@@ -103,7 +102,7 @@ sub parse {
 		$status=0;
 	    }
 	} else {
-	    die sprintf(gettext("%s: Syntax error",$ref))."\n";
+	    die wrap_ref_mod($ref, "po4a::kernelhelp", gettext("Syntax error"));
 	}
 
     	# Reinit the loop
@@ -143,7 +142,7 @@ Locale::Po4a::KernelHelp - Convert kernel configuration help from/to PO files
 =head1 DESCRIPTION
 
 Locale::Po4a::KernelHelp is a module to help the translation of
-documentation for the linux kernel configuration options into other [human]
+documentation for the Linux kernel configuration options into other [human]
 languages.
 
 =head1 STATUS OF THIS MODULE
@@ -162,13 +161,13 @@ L<Locale::Po4a::Pod(3pm)>,
 =head1 AUTHORS
 
  Denis Barbier <barbier@linuxfr.org>
- Martin Quinson <martin.quinson@tuxfamily.org>
+ Martin Quinson (mquinson#debian.org)
 
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2002 by SPI, inc.
 
 This program is free software; you may redistribute it and/or modify it
-under the terms of GPL (see COPYING file).
+under the terms of GPL (see the COPYING file).
 
 =cut
