@@ -36,8 +36,8 @@ in wrapped paragraph, and I think it's worth.
 
 =head2 Font specification
 
-The first change is about font change specifications.  In nroff, there is
-several way to specify if a given word should be written in small, bold or
+The first change is about font change specifications.  In nroff, there are
+several ways to specify if a given word should be written in small, bold or
 italics. In the text to translate, there is only one way, borrowed from the
 pod (perl online documentation) format:
 
@@ -62,7 +62,7 @@ equivalent to \f(CWtext\fP or ".CW text"
 =back
 
 Remark: The CW face is not available for all groff devices. It is not
-recommanded to use it. It is provided for your convenience.
+recommended to use it. It is provided for your convenience.
 
 =head2 Putting 'E<lt>' and 'E<gt>' in translations
 
@@ -70,6 +70,39 @@ Since these chars are used to delimit parts under font modification, you
 can't use them verbatim. Use EE<lt>ltE<gt> and EE<lt>gtE<gt> instead (as in
 pod, one more time).
 
+=head1 OPTIONS ACCEPTED BY THIS MODULE
+
+These are this module's particular options:
+
+=over 4
+
+=item B<debug>
+
+Activate debugging for some internal mechanisms of this module.
+Use the source to see which parts can be debugged.
+
+=item B<with-non_breaking_spaces> (boolean)
+
+Allow po4a to generate non breaking spaces in the PO file (when needed).
+
+Anyway, users can use non breaking spaces in the PO and are always converted to
+the roff non breaking space ('\ ').
+
+This option is disabled by default because non breaking spaces are not present
+in all character set.
+
+=item B<with-hyphen_to_minus> (boolean)
+
+Allow the automatic convertion of hyphens (in roff: '-') to minus signs ('\-').
+When enable, translators will only see/use '-' instead of '\\-' in their PO
+files.
+
+This option is enabled by default, because man pages autors and translators
+always forget to use '\-' instead of '-' (and it doesn't harm to have a minus
+sign instead of an hyphen).
+
+=back
+
 =head1 AUTHORING MAN PAGES COMPLIANT WITH PO4A::MAN
 
 This module is still very limited, and will always be, because it's not a
@@ -77,8 +110,8 @@ real nroff interpreter. It would be possible to do a real nroff
 interpreter, to allow authors to use all the existing macros, or even to
 define new ones in their pages, but we didn't want to. It would be too
 difficult, and we thought it wasn't necessary. We do think that if
-manpage authors want to see their production translated, they may have to
-adapt to ease the work of translators. 
+manpages' authors want to see their productions translated, they may have to
+adapt to ease the work of translators.
 
 So, the man parser implemented in po4a have some known limitations we are
 not really inclined to correct, and which will constitute some pitfalls
@@ -91,9 +124,9 @@ The macro set described in mdoc(7) (and widely used under BSD, IIRC) isn't
 supported at all by po4a, and won't be. It would need a completely separate
 parser for this, and I'm not inclined to do so. On my machine, there are
 only 63 pages based on mdoc, from 4323 pages. If someone implements the mdoc
-support, I'll happilly include this, though.
+support, I'll happily include this, though.
 
-=head2 Don't programm in nroff
+=head2 Don't program in nroff
 
 nroff is a complete programming language, with macro definition,
 conditionals and so on. Since this parser isn't a fully featured nroff
@@ -114,7 +147,7 @@ only because I failed to find any documentation about them. Here is the
 list of unsupported macros used on my box. Note that this list isn't
 exhaustive since the program fails on the first encountered unsupported
 macro. If you have any information about some of these macros, I'll
-happilly add support for them. Because of these macros, about 250 pages on
+happily add support for them. Because of these macros, about 250 pages on
 my box are inaccessible to po4a::man.
 
  ..               ."              .AT             .b              .bank
@@ -196,7 +229,7 @@ automatically), or have other more serious formatting change (ie, which
 chars are italics, which ones are bold, etc.).
 
 So, it seems like since ignored pages are translatable with po4a::pod and
-since wrapping changes are acceptables in most cases, the current version
+since wrapping changes are acceptable in most cases, the current version
 of po4a can translate 80% of the man pages on my machine. Moreover, most of
 the untranslatable pages could be fixed with some simple tricks given
 above. Isn't that coooool?
@@ -209,15 +242,15 @@ L<Locale::Po4a::Pod(3pm)>.
 =head1 AUTHORS
 
  Denis Barbier <barbier@linuxfr.org>
- Nicolas Franois <nicolas.francois@centraliens.net>
- Martin Quinson <martin.quinson@tuxfamily.org>
+ Nicolas François <nicolas.francois@centraliens.net>
+ Martin Quinson (mquinson#debian.org)
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2002, 2003, 2004 by SPI, inc.
+Copyright 2002, 2003, 2004, 2005 by SPI, inc.
 
 This program is free software; you may redistribute it and/or modify it
-under the terms of GPL (see COPYING file).
+under the terms of GPL (see the COPYING file).
 
 =cut
 
@@ -233,6 +266,7 @@ $VERSION=$Locale::Po4a::TransTractor::VERSION;
 @ISA = qw(Locale::Po4a::TransTractor);
 @EXPORT = qw();#  new initialize);
 use Locale::Po4a::TransTractor;
+use Locale::Po4a::Common;
 use Locale::gettext qw(dgettext);
 
 use File::Spec;
@@ -246,8 +280,6 @@ my %macro; # hash of known macro, with parsing sub. See end of this file
 # .    - a single char (e.g. B, I, R, P, 1, 2, 3, 4, etc.)
 my $FONT_RE = "\\\\f(?:\\[[^\\]]*\\]|\\(..|[^\\(\\[])";
 
-sub initialize {}
-
 #########################
 #### DEBUGGING STUFF ####
 #########################
@@ -257,6 +289,44 @@ my %debug=('splitargs' => 0, # see how macro args are separated
 	   'fonts'    => 0,  # see font modifier handling
 	   );
 
+
+######## CONFIG #########
+my %transliterations = (
+    'nbs'   => 0,     # allow generating non breaking spaces?
+    'minus' => 1      # force using minus signs instead of hyphens?
+    );
+sub initialize {
+    my $self = shift;
+    my %options = @_;
+
+    $self->{options}{'with-non_breaking_spaces'}='';
+    $self->{options}{'with-hyphen_to_minus'}='';
+    $self->{options}{'debug'}='';
+
+    foreach my $opt (keys %options) {
+        if ($options{$opt}) {
+            die wrap_mod("po4a::tex",
+                         dgettext("po4a", "Unknown option: %s"), $opt)
+                unless exists $self->{options}{$opt};
+            $self->{options}{$opt} = $options{$opt};
+        }
+    }
+
+    if (defined $options{'debug'}) {
+        foreach ($options{'debug'}) {
+            $debug{$_} = 1;
+        }
+    }
+
+    if (defined $options{'with-non_breaking_spaces'}) {
+        $transliterations{'nbs'} = $options{'with-non_breaking_spaces'};
+    }
+
+    if (defined $options{'with-hyphen_to_minus'}) {
+        $transliterations{'minus'} = $options{'with-hyphen_to_minus'};
+    }
+}
+
 # This function returns the next line of the document being parsed
 # (and its reference).
 # It overload the Transtractor shiftline to handle:
@@ -276,12 +346,31 @@ sub shiftline {
         return ($line,$ref);
     }
 
+    # Handle some escapes
+    #   * reduce the number of \ in macros
+    if ($line =~ /^\\?[.']/) {
+        # The first backslash is consumed while the macro is read.
+        $line =~ s/\\\\/\\/g;
+    }
+    #   * \\ is equivalent to \e, which is less error prone for the rest
+    #     of the module (e.g. when searching for a font : \f, whe don't
+    #     want to match \\f)
+    $line =~ s/\\\\/\\e/g;
+    #   * \. is just a dot (this can even be use to introduce a macro)
+    $line =~ s/\\\././g;
+
     chomp $line;
-    while ($line =~ /^\..*\\$/ || $line =~ /^(\.[BI])\s*$/) {
+    while ($line =~ /\\$/ || $line =~ /^(\.[BI])\s*$/) {
         my ($l2,$r2)=$self->SUPER::shiftline();
         chomp($l2);
         if ($line =~ /^(\.[BI])\s*$/) {
-            $l2 =~ s/"/\\"/g;
+            # convert " to the groff's double quote glyph; it will be
+            # converted back to " in pre_trans. It is needed because
+            # otherwise, these quotes will be taken as arguments
+            # delimiters.
+            $l2 =~ s/"/\\(dq/g;
+            # append this line to the macro, with surrounding quotes, so
+            # that the line appear as an uniq argument.
             $line .= ' "'.$l2.'"';
         } else {
             $line =~ s/\\$//;
@@ -324,10 +413,10 @@ sub shiftline {
 # The default unshiftline from Transtractor may fail because shiftline
 # is overloaded
 sub unshiftline {
-    die sprintf(dgettext("po4a",
-        "po4a::man: The unshiftline is not supported for the man module.\n".
-        "po4a::man: Please send a bug report with the groff page that generated\n".
-        "po4a::man: this error."))."\n";
+    die wrap_mod("po4a::man", dgettext("po4a",
+	"The unshiftline is not supported for the man module. ".
+	"Please send a bug report with the groff page that generated ".
+	"this error."));
 }
 
 ###############################################
@@ -344,14 +433,22 @@ sub pushmacro {
 	# Adding extra quotes as in:
 	# .IP "CATALOG\ "ddddddddddddd""
 	# results in two args: 'CATALOG\ ' and 'ddddddddddddd""'
-  
-	$self->pushline(join(" ",map { defined $_ ? 
-					 ($_ eq '0' ? "0" 
-					            : ( length($_) && m/([^\\] |^ )/ ? "\"$_\"" 
-							                   : "$_"||'""'
-						      )
-				         ) : ''
-				      } @_)."\n");
+	$self->pushline(join(" ",map {
+		# Replace double quotes by \(dq (double quotes could be
+		# taken as an argument delimiter).
+		# Only quotes not preceded by \ are taken into account
+		# (\" introduces a comment).
+		s/(?<!\\)"/\\\(dq/g if (defined $_);
+
+		defined $_ ? (
+			length($_)?
+			    (m/([^\\] |^ )/ ? "\"$_\"" : "$_")
+			    # Quote arguments that contain a space.
+			    # (not needd for non breaknig spaces, i.e.
+			    # spaces preceded by '\')
+			    :'""' # empty argument
+		) : '' # no argument
+	    } @_)."\n");
     } else {
 	$self->pushline("\n");
     }
@@ -359,22 +456,27 @@ sub pushmacro {
 sub this_macro_needs_args {
     my ($macroname,$ref,$args)=@_;
     unless (length($args)) {
-	die sprintf(dgettext("po4a",
-		"po4a::man: %s: macro %s called without arguments.\n".
-		"po4a::man: Even if placing the macro arguments on the next line is authorized\n".
-		"po4a::man: by man(7), handling this would make the po4a parser too complicate.\n".
-		"po4a::man: Please simply put the macro args on the same line."
-		),$ref,$macroname)."\n";
+	die wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+		"macro %s called without arguments. ".
+		"Even if placing the macro arguments on the next line is authorized ".
+		"by man(7), handling this would make the po4a parser too complicate. ".
+		"Please simply put the macro args on the same line."
+		), $macroname);
     }
 }
 
 sub pre_trans {
     my ($self,$str,$ref,$type)=@_;
-    # Preformating, so that translators don't see 
+    # Preformatting, so that translators don't see 
     # strange chars
     my $origstr=$str;
     print STDERR "pre_trans($str)="
 	if ($debug{'pretrans'});
+
+    # Note: if you want to implement \c support, the gdb man page is your playground
+    die wrap_ref_mod($ref, "po4a::man", dgettext("po4a","Escape sequence \\c encountered. This is not completely handled yet."))
+	if ($str =~ /\\c/);
+
     $str =~ s/>/E<gt>/sg;
     $str =~ s/</E<lt>/sg;
     $str =~ s/EE<lt>gt>/E<gt>/g; # could be done in a smarter way?
@@ -384,22 +486,39 @@ sub pre_trans {
         set_regular("B");
     }
     $str = do_fonts($str);
-    if (defined $self->{type} && $self->{type} =~ m/^(SH|HP|SS)$/) {
+    if (defined $self->{type} && $self->{type} =~ m/^(SH|SS)$/) {
         set_regular("R");
     }
 
+    # After the simplification, the first char can be a \n.
+    # Simply push these newlines before the translation, but make sure the
+    # resulting string is not empty (or an additionnal line will be
+    # added).
+    if ($str =~ /^(\n+)(.+)$/s) {
+        $self->pushline($1);
+        $str = $2;
+    }
+
     # Kill minus sign/hyphen difference.
     # Aestetic of printed man pages may suffer, but:
     #  * they are translator-unfriendly
     #  * they break when using utf8 (for obscure reasons)
     #  * they forbid the searches, since keybords don't have hyphen key
     #  * they forbid copy/paste, since options need minus sign, not hyphen
-    $str =~ s|\\-|-|sg;
+    if ($transliterations{'minus'}) {
+        $str =~ s|\\-|-|sg;
+    }
     # Groff bestiary
     $str =~ s/\\\*\(lq/``/sg;
     $str =~ s/\\\*\(rq/''/sg;
+    $str =~ s/\\\(dq/"/sg;
     # Change groff non-breaking space to ascii one
-    $str =~ s|\\ |\xA0|sg;
+    if ($transliterations{'nbs'}) {
+        $str =~ s|\\ |\xA0|sg;
+    } else {
+        # some \xA0 may have been added during the parsing
+        $str =~ s|\xA0|\\ |sg;
+    }
 
     print STDERR "$str\n" if ($debug{'pretrans'});
     return $str;
@@ -412,17 +531,32 @@ sub post_trans {
     print STDERR "post_trans($str)="
 	if ($debug{'postrans'});
 
-    # Post formating, so that groff see the strange chars
-    $str =~ s|\\-|-|sg; # in case the translator added some of them manually
-    # change hyphens to minus signs
-    # (this shouldn't be done for \s-<number> font size modifiers)
-    # nor on .so/.mso args
-    unless (defined $self->{type} && $self->{type} =~ m/^m?so$/) {
-        $str =~ s/(?<!\\s)-/\\-/sg; # (?<!pattern) means "not preceded by pattern"
+    # Post formatting, so that groff see the strange chars
+    if ($transliterations{'minus'}) {
+        $str =~ s|\\-|-|sg; # in case the translator added some of them manually
+        # change hyphens to minus signs
+        # (this shouldn't be done for \s-<number> font size modifiers)
+        # nor on .so/.mso args
+        unless (defined $self->{type} && $self->{type} =~ m/^m?so$/) {
+            $str =~ s/(?<!\\s)-/\\-/sg; # (?<!pattern) means "not preceded by pattern"
+        }
     }
 
-    # No . on first char, or nroff will think it's a macro
-    $str =~ s/\n([.'"])/ $1/mg; #'
+    # No . or ' on first char, or nroff will think it's a macro
+    # * at the beginning of a paragraph, add \& (zero width space) at
+    #   the beginning of the line
+    unless (defined $self->{type} && $self->{type} =~ m/^(TS)$/) {
+        # This doesn't work after a TS (this macros shift
+        # lines, which may contain macros)
+        $str =~ s/^((?:
+                       (?:CW|[RBI])<
+                      |$FONT_RE
+                    )?
+                    [.']
+                   )/\\&$1/mgx;
+    }
+    # * degraded mode, doesn't work for the first line of a paragraph
+    $str =~ s/\n([.'])/ $1/mg;
 
     # Change ascii non-breaking space to groff one
     $str =~ s|\xA0|\\ |sg;
@@ -446,7 +580,7 @@ sub post_trans {
 	    $done .= $first  if ($lvl > 0);
 	    $rest=substr($rest,1);
 	}
-	die sprintf("po4a::man: %s: ".dgettext("po4a","Unbalanced '<' and '>' in '%s'"),$ref||$self->{ref},$transstr)."\n"
+	die wrap_ref_mod($ref||$self->{ref}, "po4a::man", dgettext("po4a","Unbalanced '<' and '>'"))
 	    if ($lvl > 0);
 	# Return to the regular font
 	$done .= "\\fP$rest";
@@ -534,15 +668,12 @@ sub parse{
 	chomp($line);
 	$self->{ref}="$ref";
 #	print STDERR "LINE=$line<<\n";
-	die sprintf("po4a::man: %s: ".dgettext("po4a","Escape sequence \\c encountered. This is not handled yet.")
-			    ,$ref)."\n"
-	    if ($line =~ /\\c/);
 
 
-	if ($line =~ /^\./) {
-	    die sprintf("po4a::man: ".dgettext("po4a","Unparsable line: %s"),$line)."\n"
-		unless ($line =~ /^(\.+\\*?)(\\\")(.*)/ ||
-			$line =~ /^(\.)(\S*)(.*)/);
+	if ($line =~ /^[.']/) {
+	    die wrap_mod("po4a::man", dgettext("po4a", "Unparsable line: %s"), $line)
+		unless ($line =~ /^([.']+\\*?)(\\\")(.*)/ ||
+			$line =~ /^([.'])(\S*)(.*)/);
 	    my $arg1=$1;
 	    $arg1 .= $2;
 	    my $macro=$2;
@@ -555,7 +686,7 @@ sub parse{
 	    push @args, splitargs($ref,$arguments);
 
 
-	    if ($paragraph) {
+	    if (length($paragraph)) {
 		do_paragraph($self,$paragraph,$wrapped_mode);
 		$paragraph="";
 		$wrapped_mode = $wrapped_mode eq 'NO' ? 'YES' : $wrapped_mode;
@@ -566,35 +697,35 @@ sub parse{
 	    if ($macro eq '\"' ||
 		$macro eq '"') {
 		if ($line =~ /Pod::Man/) {
-		    warn dgettext("po4a","This file was generated with Pod::Man. Translate the pod file with the pod module of po4a.")."\n";
-		    exit 0;
+		    warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with Pod::Man. Translate the pod file with the pod module of po4a."));
+		    exit 254;
 		} elsif ($line =~ /generated by help2man/)    {
-		    warn dgettext("po4a","This file was generated with help2man. Translate the source file with the regular gettext.")."\n";
-		} elsif ($line =~ /with docbook-to-man/)      { 
-		    warn dgettext("po4a","This file was generated with docbook-to-man. Translate the source file with the sgml module of po4a.")."\n";
-		    exit 0;
-		} elsif ($line =~ /generated by docbook2man/) { 
-		    warn dgettext("po4a","This file was generated with docbook2man. Translate the source file with the sgml module of po4a.")."\n";
-		    exit 0;
-		} elsif ($line =~ /created with latex2man/)   { 
-		    warn sprintf(dgettext("po4a",
-					"This file was generated with %s.\n".
-					"You should translate the source file, but continuing anyway."
-					),"latex2man")."\n";
+		    warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with help2man. Translate the source file with the regular gettext."));
+		} elsif ($line =~ /with docbook-to-man/)      {
+		    warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with docbook-to-man. Translate the source file with the sgml module of po4a."));
+		    exit 254;
+		} elsif ($line =~ /generated by docbook2man/) {
+		    warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with docbook2man. Translate the source file with the sgml module of po4a."));
+		    exit 254;
+		} elsif ($line =~ /created with latex2man/)   {
+		    warn wrap_mod("po4a::man", dgettext("po4a",
+			"This file was generated with %s. ".
+			"You should translate the source file, but continuing anyway."
+			),"latex2man");
 		} elsif ($line =~ /Generated by db2man.xsl/)  { 
-		    warn dgettext("po4a","This file was generated with db2man.xsl. Translate the source file with the xml module of po4a.")."\n";
-		    exit 0;
+		    warn wrap_mod("po4a::man", dgettext("po4a","This file was generated with db2man.xsl. Translate the source file with the xml module of po4a."));
+		    exit 254;
 		} elsif ($line =~ /generated automatically by mtex2man/)  {
-		    warn sprintf(dgettext("po4a",
-					"This file was generated with %s.\n".
-					"You should translate the source file, but continuing anyway."
-					),"mtex2man")."\n";
+		    warn wrap_mod("po4a::man", dgettext("po4a",
+			"This file was generated with %s. ".
+			"You should translate the source file, but continuing anyway."
+			),"mtex2man");
 		} elsif ($line =~ /THIS FILE HAS BEEN AUTOMATICALLY GENERATED.  DO NOT EDIT./ ||
 		         $line =~ /DO NOT EDIT/i || $line =~ /generated/i) {
-		    warn sprintf(dgettext("po4a",
-					"This file contains the line '%s'.\n".
-					"You should translate the source file, but continuing anyway."
-					),"$line")."\n";
+		    warn wrap_mod("po4a::man", dgettext("po4a",
+			"This file contains the line '%s'. ".
+			"You should translate the source file, but continuing anyway."
+			),$line);
 		}
 	    }
 
@@ -631,8 +762,8 @@ sub parse{
 	    # Special case:
 	    #  .Dd => Indicates that this is a mdoc page
 	    if ($macro eq 'Dd') {
-		die "po4a::man: ".dgettext("po4a",
-			"This page seems to be a mdoc(7) formated one. This is not supported (yet).")."\n";
+		die wrap_mod("po4a::man", dgettext("po4a",
+		    "This page seems to be a mdoc(7) formatted one. This is not supported (yet)."));
 	    }
 		
 	    unshift @args,$self;
@@ -643,32 +774,38 @@ sub parse{
 		&{$macro{$macro}}(@args);
 	    } else {
 		$self->pushline($line."\n");
-		die sprintf(dgettext("po4a",
-		    "po4a::man: Unknown macro '%s' (at %s).\n".
-		    "po4a::man: Remove it from the document, or provide a patch to the po4a team."
-				),$line,$ref)."\n";
+		die wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+		    "Unknown macro '%s'. Remove it from the document, or provide a patch to the po4a team."), $line);
 	    }
 
-	} elsif ($line =~ /^( +)([^.].*)/) {
+	} elsif ($line =~ /^( +)([^. ].*)/) {
+	    # (Lines containing only spaces are handled as empty lines)
 	    # Not a macro, but not a wrapped paragraph either
-	    $wrapped_mode='NO';
+	    $wrapped_mode = $wrapped_mode eq 'YES' ? 'NO' : $wrapped_mode;
 	    $paragraph .= $line."\n";
-	} elsif ($line =~ /^([^.].*)/) {
-            # special case: the line is entirely a comment, keep the
-            # comment.
-            # NOTE: comment could also be found in the middle of a line.
-            # From info groff:
-            # Escape: \": Start a comment.  Everything to the end of the
-            # input line is ignored.
-            if ($line =~ /^\\"/) {
+	} elsif ($line =~ /^([^.].*)/ && $line !~ /^ *$/) {
+	    # (Lines containing only spaces are handled latter as empty lines)
+	    if ($line =~ /^\\"/) {
+		# special case: the line is entirely a comment, keep the
+		# comment.
+		# NOTE: comment could also be found in the middle of a line.
+		# From info groff:
+		# Escape: \": Start a comment.  Everything to the end of the
+		# input line is ignored.
 		$self->pushline($line."\n");
 		goto LINE;
-            }
-	    # Not a macro
-	    $paragraph .= $line."\n";
-	} else { #empty line
-	    do_paragraph($self,$paragraph,$wrapped_mode);
-	    $paragraph="";
+            } else {
+		# Not a macro
+		# * first, try to handle some "output line continuation" (\c)
+		$paragraph =~ s/\\c *(($FONT_RE)?)\n?$/$1/s;
+		# * append the line to the current paragraph
+		$paragraph .= $line."\n";
+	    }
+	} else { #empty line, or line containing only spaces
+	    if (length($paragraph)) {
+	        do_paragraph($self,$paragraph,$wrapped_mode);
+	        $paragraph="";
+	    }
 	    $wrapped_mode = $wrapped_mode eq 'NO' ? 'YES' : $wrapped_mode;
 	    $self->pushline($line."\n");
 	}
@@ -678,7 +815,7 @@ sub parse{
 	undef $self->{type};
     }
 
-    if ($paragraph) {
+    if (length($paragraph)) {
 	do_paragraph($self,$paragraph,$wrapped_mode);
 	$wrapped_mode = $wrapped_mode eq 'NO' ? 'YES' : $wrapped_mode;
 	$paragraph="";
@@ -714,6 +851,8 @@ sub splitargs {
             $buffer .= " ".$elem;
             print STDERR "Continuation of a quote\n"
                 if ($debug{'splitargs'});
+            # Inside a quoted argument, "" indicates a double quote
+            while ($buffer =~ s/^"([^"]*?)""/"$1\\(dq/) {}
             # print "buffer=$buffer.\n";
             if ($buffer =~ m/^"(.*)"(.+)$/) {
                 print STDERR "End of quote, with stuff after it\n"
@@ -735,12 +874,8 @@ sub splitargs {
                 print STDERR "End of an escaped sequence\n"
                     if ($debug{'splitargs'});
                 unless(length($elem)){
-                    die sprintf(dgettext("po4a",
-                                "po4a::man: %s: Escaped space at the end of macro arg. With high\n".
-                                "po4a::man: probability, it won't do the trick with po4a (because of\n".
-                                "po4a::man: wrapping). You may want to remove it and use the .nf/.fi groff\n".
-                                "po4a::man: macro to control the wrapping."),
-                                 $ref)."\n";
+                    die wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+			"Escaped space at the end of macro arg. With high probability, it won't do the trick with po4a (because of wrapping). You may want to remove it and use the .nf/.fi groff macro to control the wrapping."));
                 }
                 $buffer =~ s/\xA0/\\ /g;
                 push @args,$buffer;
@@ -751,6 +886,8 @@ sub splitargs {
             print STDERR "Quoted, no space\n"
                 if ($debug{'splitargs'});
             my $a = $1;
+            # Inside a quoted argument, "" indicates a double quote
+            $a =~ s/""/\\(dq/g;
             $a =~ s/\xA0/\\ /g;
             push @args,$a;
         } elsif ($elem =~ m/^"/) { #") {
@@ -761,7 +898,7 @@ sub splitargs {
             print STDERR "escaped space after $1\n"
                 if ($debug{'splitargs'});
             # escaped space
-            $buffer = ($buffer?$buffer:'').$1." ";
+            $buffer = (length($buffer)?$buffer:'').$1." ";
             $escaped = 1;
         } else {
             print STDERR "Unquoted arg, nothing to declare\n"
@@ -770,7 +907,9 @@ sub splitargs {
             $buffer="";
         }
     }
-    if ($buffer) {
+    if (length($buffer)) {
+        # Inside a quoted argument, "" indicates a double quote
+        while ($buffer =~ s/^"([^"]*?)""/"$1\\(dq/) {}
         $buffer=~ s/"//g;
         $buffer =~ s/\xA0/\\ /g;
         push @args,$buffer;
@@ -882,7 +1021,7 @@ sub splitargs {
                     $str .= "\\f".$elem;
                 }
             } else {
-                die sprintf("po4a::man: ".dgettext("po4a","Unsupported font in: '%s'."),$elem)."\n";
+                die wrap_mod("po4a::man", dgettext("po4a", "Unsupported font in: '%s'."), $elem);
             }
         }
         # Do some simplification (they don't change the font stack)
@@ -1016,7 +1155,23 @@ $macro{'TH'}= sub {
 };
 
 # .SS t    Subheading t (like .SH, but used for a subsection inside a section).
-$macro{'SS'}=$macro{'SH'}=\&translate_joined;
+$macro{'SS'}=$macro{'SH'}=sub {
+    if (!defined $_[2]) {
+        # The argument is on the next line.
+        my ($self,$macroname) = (shift,shift);
+        my ($l2,$ref2) = $self->shiftline();
+        if ($l2 =~/^\./) {
+            $self->SUPER::unshiftline($l2,$ref2);
+        } else {
+            chomp($l2);
+            $self->pushmacro($macroname,
+                             $self->t($l2));
+        }
+        return;
+    } else {
+        return translate_joined(@_);
+    }
+};
 
 $macro{'SM'}=\&translate_joined;
 
@@ -1060,7 +1215,17 @@ $macro{'TP'}=sub {
 	($l2,$ref2) = $self->shiftline();
 	chomp($l2);
     }
-    $self->pushline($self->t($l2)."\n");
+    if ($l2 =~/^[.']/) {
+	# If the line after a .TP is a macro,
+	# let the parser do it's job.
+	# Note: use Transtractor unshiftline for now. This may require an
+	#       implementation of the man module's own implementation.
+	#       This may be a problem if, for example, the line resulted
+	#       of a line continuation.
+	$self->SUPER::unshiftline($l2,$ref2);
+    } else {
+	$self->pushline($self->t($l2)."\n");
+    }
 
     # From info groff:
     # Note that neither font shape nor font size of the label [i.e. argument
@@ -1075,14 +1240,11 @@ $macro{'TP'}=sub {
 #                graph's lines are indented).
 #
 $macro{'HP'}=sub {
-    my $self=shift;
-    my ($line,$l2,$ref2);
-    $line .= $_[0] if defined($_[0]);
-    $line .= ' '.$_[1] if defined($_[1]);
-    $line .= "\n";
-    ($l2,$ref2) = $self->shiftline();
-    chomp($l2);
-    $self->pushline($line.$self->t($l2)."\n");
+    untranslated(@_);
+
+    # From info groff:
+    # Font size and face are reset to their default values.
+    set_font("R");
 };
 
 # Indented Paragraph Macros
@@ -1165,7 +1327,7 @@ $macro{'bp'}=\&untranslated;
 $macro{'ad'}=\&untranslated;
 # .de macro Define or redefine macro until .. is encountered.
 $macro{'de'}=sub {
-    die "po4a::man: ".dgettext("po4a","This page defines a new macro with '.de'. Since po4a is not a real groff parser, this is not supported.")."\n";
+    die wrap_mod("po4a::man", dgettext("po4a", "This page defines a new macro with '.de'. Since po4a is not a real groff parser, this is not supported."));
 };
 # .ds stringvar anything
 #                 Set stringvar to anything.
@@ -1194,9 +1356,8 @@ $macro{'hy'}=$macro{'hym'}=$macro{'hys'}=\&untranslated;
 # .ie cond anything  If cond then anything else goto .el.
 # .if cond anything  If cond then anything; otherwise do nothing.
 $macro{'ie'}=$macro{'if'}=sub {
-    die sprintf("po4a::man: ".
-	  dgettext("po4a","This page uses conditionals with '%s'. Since po4a is not a real groff parser, this is not supported."),
-	        $_[1])."\n";
+    die wrap_mod("po4a::man", dgettext("po4a",
+    	"This page uses conditionals with '%s'. Since po4a is not a real groff parser, this is not supported."), $_[1]);
 };
 # .in  N    Change indent according to N (default scaling indicator m).
 $macro{'in'}=\&untranslated;
@@ -1235,9 +1396,8 @@ $macro{'ps'}=\&untranslated;
 # .so filename Include source file.
 # .mso groff variant of .so (other search path)
 $macro{'so'}= $macro{'mso'} = sub {
-    die "po4a::man: ".sprintf(
-      dgettext("po4a","This page includes another file with '%s'. This is not supported yet, but will soon."),
-	$_[1])."\n",;
+    die wrap_mod("po4a::man", dgettext("po4a",
+	"This page includes another file with '%s'. This is not supported yet, but will soon."), $_[1]);
 };
 # .sp     Skip one line vertically.
 # .sp N   Space  vertical distance N
@@ -1335,10 +1495,8 @@ $macro{'ce'}=$macro{'ul'}=$macro{'cu'}=sub {
         } else {
 # All of these are not handled yet because the number of line may change
 # during the translation
-            die sprintf("po4a::man: ".
-               dgettext("po4a","This page uses the '%s' request with the ".
-                               "number of lines in argument. This is not ".
-                               "supported yet.\n"),$_[0])."\n";
+            die wrap_mod("po4a::man", dgettext("po4a",
+		"This page uses the '%s' request with the number of lines in argument. This is not supported yet."), $_[0]);
         }
     } else {
 	$self->pushmacro($_[0]);
@@ -1352,10 +1510,8 @@ $macro{'ce'}=$macro{'ul'}=$macro{'cu'}=sub {
 $macro{'ec'}=sub {
     my $self=shift;
     if (defined $_[1]) {
-        die sprintf("po4a::man: ".
-           dgettext("po4a","This page uses the '%s' request.  This request ".
-                           "is only supported when no argument is ".
-                           "provided.\n"),$_[0])."\n";
+        die wrap_mod("po4a::man", dgettext("po4a",
+	    "This page uses the '%s' request. This request is only supported when no argument is provided."), $_[0]);
     } else {
         $self->pushmacro($_[0]);
     }
