@@ -1,7 +1,5 @@
 # Locale::Po4a::Po -- manipulation of po files 
-# $Id: Po.pm,v 1.29 2004/11/05 20:33:51 barbier Exp $
-#
-# Copyright 2002 by Martin Quinson <Martin.Quinson@ens-lyon.fr>
+# $Id: Po.pm,v 1.37 2005/02/27 22:53:53 mquinson Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -59,6 +57,7 @@ require Exporter;
 package Locale::Po4a::Po;
 
 use Locale::Po4a::TransTractor;
+use Locale::Po4a::Common;
 
 use 5.006;
 use strict;
@@ -145,14 +144,14 @@ catalog.
 sub read{
     my $self=shift;
     my $filename=shift 
-	|| croak ("po4a::po: ".dgettext("po4a","Please provide a non-nul filename")."\n");
+	|| croak wrap_mod("po4a::po", dgettext("po4a", "Please provide a non-null filename"));
 
     my $fh;
     if ($filename eq '-') {
 	$fh=*STDIN;
     } else {
 	open $fh,"<$filename" 
-	  || croak ("po4a::po: ".sprintf(dgettext("po4a","Can't read from %s: %s"),$filename,$!)."\n");
+	  || croak wrap_mod("po4a::po", dgettext("po4a", "Can't read from %s: %s"), $filename, $!);
     }
 
     ## Read paragraphs line-by-line
@@ -191,8 +190,7 @@ sub read{
 	    } elsif ($line =~ /^(".*")/) { # continuation of a line
 	        $buffer .= "\n$1";
 	    } else {
-	        warn sprintf(dgettext("po4a","Strange line at line %s: -->%s<--"),
-			     $linenum,$line)."\n";
+	        warn wrap_ref_mod("$filename:$linenum", "po4a::po", dgettext("po4a", "Strange line: -->%s<--"), $line);
 	    }
 	}
 	$linenum++;
@@ -232,7 +230,7 @@ sub write{
 	      if (length ($dir) && ! -e $dir);
 	}
 	open $fh,">$filename" 
-	    || croak (sprintf((dgettext("po4a","Can't write to %s: %s"),$filename,$!))."\n");
+	    || croak wrap_mod("po4a::po", dgettext("po4a", "Can't write to %s: %s"), $filename, $!);
     }
 
     print $fh "".format_comment(unescape_text($self->{header_comment}),"") 
@@ -291,17 +289,17 @@ sub gettextize {
     my $pores=Locale::Po4a::Po->new();
 
     if ($poorig->count_entries() > $potrans->count_entries()) {
-	warn sprintf(dgettext("po4a",
-		"po4a gettextize: Original have more strings that the translation (%d>%d).\n".
-		"po4a gettextize: Please fix it by editing the translated version to add some dummy entry."),
-		    $poorig->count_entries() , $potrans->count_entries())."\n";
+	warn wrap_mod("po4a gettextize", dgettext("po4a",
+	    "Original has more strings than the translation (%d>%d). ".
+	    "Please fix it by editing the translated version to add some dummy entry."),
+		$poorig->count_entries() , $potrans->count_entries());
     } elsif ($poorig->count_entries() < $potrans->count_entries()) {
-	warn sprintf(dgettext("po4a",
-		"po4a gettextize: Original have less strings that the translation (%d<%d).\n".
-		"po4a gettextize: Please fix it by removing the extra entry from the translated file.\n".
-		"po4a gettextize: You may need an addendum (cf po4a(7)) to reput the chunk in place after gettextization.\n".
-	        "po4a gettextize: A possible cause is that a text dupplicated in the original is not translated the same way each time. Remove one of the translations, and you're fine."),
-		    $poorig->count_entries() , $potrans->count_entries())."\n";
+	warn wrap_mod("po4a gettextize", dgettext("po4a",
+	    "Original has less strings than the translation (%d<%d). ".
+	    "Please fix it by removing the extra entry from the translated file. ".
+	    "You may need an addendum (cf po4a(7)) to reput the chunk in place after gettextization. ".
+	    "A possible cause is that a text duplicated in the original is not translated the same way each time. Remove one of the translations, and you're fine."),
+		$poorig->count_entries(), $potrans->count_entries());
     }
 
     if ( $poorig->get_charset =~ /^utf-8$/i ) {
@@ -337,10 +335,10 @@ sub gettextize {
 	#
 	# Make sure the type of both string exist
 	#
-	die sprintf("Internal error in gettextization: type of original string number %s isn't provided\n",$o)."\n"
+	die wrap_mod("po4a gettextize", "Internal error: type of original string number %s isn't provided", $o)
 	    if ($typeorig eq '');
 	
-	die sprintf("Internal error in gettextization: type of translated string number %s isn't provided\n",$o)."\n"
+	die wrap_mod("po4a gettextize", "Internal error: type of translated string number %s isn't provided", $o)
 	    if ($typetrans eq '');
 
 	#
@@ -348,16 +346,16 @@ sub gettextize {
 	#
 	if ($typeorig ne $typetrans){
 	    $pores->write("/tmp/gettextization.failed.po");
-	    die sprintf(dgettext("po4a",
+	    die wrap_msg(dgettext("po4a",
 	    	"po4a gettextization: Structure disparity between original and translated files:\n".
 		"msgid (at %s) is of type '%s' while\n".
 		"msgstr (at %s) is of type '%s'.\n".
 		"Original text: %s\n".
 		"Translated text: %s\n".
 	        "(result so far dumped to /tmp/gettextization.failed.po)"),
-			$reforig,$typeorig,$reftrans,$typetrans,$orig,$trans)."\n";
+		    $reforig, $typeorig, $reftrans, $typetrans, $orig, $trans);
 	}
-	
+
 	# 
 	# Push the entry
 	#
@@ -401,7 +399,7 @@ sub filter {
 	for (1..$pos){
 	    $space .= ' ';
 	}
-	die (sprintf(dgettext("po4a",$fmt)."\n",@_)."$filter\n$space^ HERE\n");;
+	die wrap_msg("$fmt\n$filter\n$space^ HERE");
     }
     sub showmethecode {
 	return unless $debug{'filter'};
@@ -546,13 +544,13 @@ sub filter {
     $code .= "; }";
     print STDERR "CODE = $code\n";
     eval $code;
-    die (sprintf(dgettext("po4a","Eval failure: %s")."\n",$@)) 
+    die wrap_mod("po4a::po", dgettext("po4a", "Eval failure: %s"), $@)
       if $@;
-    
+
     for (my $cpt=(0) ;
 	 $cpt<$self->count_entries();
 	 $cpt++) {
-	
+
 	my ($msgid,$ref,$msgstr,$flags,$type,$comment,$automatic);
 
 	$msgid = $self->msgid($cpt);
@@ -610,7 +608,7 @@ Request the translation of the string given as argument in the current catalog.
 The function returns the original (untranslated) string if the string was not
 found.
 
-After the string to translate, you can pass an hash of extra
+After the string to translate, you can pass a hash of extra
 arguments. Here are the valid entries:
 
 =over
@@ -673,9 +671,9 @@ sub gettext {
 
 =item stats_get()
 
-Returns stats about the hit ratio of gettext since the last time that
+Returns statistics about the hit ratio of gettext since the last time that
 stats_clear() was called. Please note that it's not the same
-statistics than the one printed by msgfmt --statistic. Here, it's stats
+statistics than the one printed by msgfmt --statistic. Here, it's statistics
 about recent usage of the po file, while msgfmt reports the status of the
 file.  Example of use:
 
@@ -689,7 +687,7 @@ file.  Example of use:
 sub stats_get() {
     my $self=shift;
     my ($h,$q)=($self->{gettexthits},$self->{gettextqueries});
-    my $p = ($q == 0 ? 0 : int($h/$q*10000)/100);
+    my $p = ($q == 0 ? 100 : int($h/$q*10000)/100);
 
 #    $p =~ s/\.00//;
 #    $p =~ s/(\..)0/$1/;
@@ -699,7 +697,7 @@ sub stats_get() {
 
 =item stats_clear()
 
-Clears the stats about gettext hits.
+Clears the statistics about gettext hits.
 
 =cut
 
@@ -718,7 +716,7 @@ sub stats_clear {
 =item push(%)
 
 Push a new entry at the end of the current catalog. The arguments should
-form an hash table. The valid keys are :
+form a hash table. The valid keys are:
 
 =over 4
 
@@ -733,8 +731,8 @@ the translation.
 =item reference
 
 an indication of where this string was found. Example: file.c:46 (meaning
-in 'file.c' at line 46). It can be a space separated list in case of
-multiple occurences.
+in 'file.c' at line 46). It can be a space-separated list in case of
+multiple occurrences.
 
 =item comment
 
@@ -773,7 +771,7 @@ gives to translators some context about the strings to translate.
 =item wrap
 
 boolean indicating whether whitespaces can be mangled in cosmetic
-reformatings. If true, the string is canonized before use.
+reformattings. If true, the string is canonized before use.
 
 This information is written to the po file using the 'wrap' or 'no-wrap' flag.
 
@@ -847,7 +845,7 @@ sub push_raw {
     }
     
     if (defined($self->{po}{$msgid})) {
-        warn sprintf(dgettext("po4a","msgid defined twice: %s"),$msgid) if (0); # FIXME: put a verbose stuff
+        warn wrap_mod("po4a::po", dgettext("po4a","msgid defined twice: %s"), $msgid) if (0); # FIXME: put a verbose stuff
 	if ($msgstr && $self->{po}{$msgid}{'msgstr'} 
 	    && $self->{po}{$msgid}{'msgstr'} ne "$msgstr") {
 	    my $txt=quote_text($msgid);
@@ -858,14 +856,14 @@ sub push_raw {
 		 format_comment(". ",$reference).
 		 quote_text($msgstr));
 
-	    warn sprintf(dgettext("po4a",
+	    warn wrap_msg(dgettext("po4a",
 	    	"Translations don't match for:\n".
 		"%s\n".
 		"-->First translation:\n".
 		"%s\n".
 		" Second translation:\n".
 		"%s\n".
-		" Old translation discarded."),$txt,$first,$second)."\n";
+		" Old translation discarded."),$txt,$first,$second);
 	}
     }
     $self->{po}{$msgid}{'reference'} = (defined($self->{po}{$msgid}{'reference'}) ? 
@@ -931,7 +929,7 @@ sub msgid($$) {
 
 =item get_charset()
 
-This returns the character set specified in the po header. If it hasn't been
+Returns the character set specified in the po header. If it hasn't been
 set, it will return "CHARSET".
 
 =cut
@@ -972,16 +970,35 @@ sub unescape_text {
     print STDERR "\nunescape [$text]====" if $debug{'escape'};
     $text = join("",split(/\n/,$text));
     $text =~ s/\\"/"/g;
-    $text =~ s/([^\\])\\n/$1\n/g;
-    $text =~ s/^\\n/\n/mg;
-    $text =~ s/([^\\])\\t/$1\t/g;
+    # unescape newlines
+    #   NOTE on \G:
+    #   The following regular expression introduce newlines.
+    #   Thus, ^ doesn't match all beginnings of lines.
+    #   \G is a zero-width assertion that matches the position
+    #   of the previous substitution with s///g. As every 
+    #   substitution ends by a newline, it always matches a
+    #   position just after a newline.
+    $text =~ s/(           # $1:
+                (\G|[^\\]) #    beginning of the line or any char
+                           #    different from '\'
+                (\\\\)*    #    followed by any even number of '\'
+               )\\n        # and followed by an escaped newline
+              /$1\n/sgx;   # single string, match globally, allow comments
+    # unescape tabulations
+    $text =~ s/(          # $1:
+                (^|[^\\]) #    beginning of the line or any char
+                          #    different from '\'
+                (\\\\)*   #    followed by any even number of '\'
+               )\\t       # and followed by an escaped tabulation
+              /$1\t/mgx;  # multilines string, match globally, allow comments
+    # and unescape the escape character
     $text =~ s/\\\\/\\/g;
     print STDERR ">$text<\n" if $debug{'escape'};
 
     return $text;
 }
 
-# transforme the string to its representation as it should be written in po files
+# transform the string to its representation as it should be written in po files
 sub escape_text {
     my $text = shift;
     
@@ -1004,8 +1021,14 @@ sub quote_text {
   return '""' unless defined($string) && length($string);
 
   print STDERR "\nquote [$string]====" if $debug{'quote'};
-  $string =~ s/([^\\])\\n/$1!!DUMMYPOPM!!/gm;
-  $string =~ s|!!DUMMYPOPM!!|\\n\n|gm;
+  # break lines on newlines, if any
+  # see unescape_text for an explanation on \G
+  $string =~ s/(           # $1:
+                (\G|[^\\]) #    beginning of the line or any char
+                           #    different from '\'
+                (\\\\)*    #    followed by any even number of '\'
+               \\n)        # and followed by an escaped newline
+              /$1\n/sgx;   # single string, match globally, allow comments
   $string = wrap($string);
   my @string = split(/\n/,$string);
   $string = join ("\"\n\"",@string);
@@ -1025,6 +1048,8 @@ sub unquote_text {
   $string =~ s/^""\\n//s;
   $string =~ s/^"(.*)"$/$1/s;
   $string =~ s/"\n"//gm;
+  # Note: an even number of '\' could precede \\n, but I could not build a
+  # document to test this
   $string =~ s/([^\\])\\n\n/$1!!DUMMYPOPM!!/gm;
   $string =~ s|!!DUMMYPOPM!!|\\n|gm;
   print STDERR ">$string<\n" if $debug{'quote'};
@@ -1032,15 +1057,14 @@ sub unquote_text {
 }
 
 # canonize the string: write it on only one line, changing consecutive whitespace to
-# only on space.
+# only one space.
 # Warning, it changes the string and should only be called if the string is plain text
 sub canonize {
     my $text=shift;
     print STDERR "\ncanonize [$text]====" if $debug{'canonize'};
     $text =~ s/^ *//s;
-    $text =~ s/([^\\])\n/$1  /gm;
-    $text =~ s/ \n/ /gm;
-    $text =~ s/([^\\])\n/$1 /gm;
+    # if ($text eq "\n"), it messed up the first string (header)
+    $text =~ s/\n/  /gm if ($text ne "\n");
     $text =~ s/([.)])  +/$1  /gm;
     $text =~ s/([^.)])  */$1 /gm;
     $text =~ s/ *$//s;
@@ -1057,7 +1081,7 @@ sub wrap {
     my @lines=split(/\n/,"$text");
     my $res="";
     my $first=1;
-    while (my $line=shift @lines) {
+    while (defined(my $line=shift @lines)) {
 	if ($first && length($line) > $col - 10) {
 	    unshift @lines,$line;
 	    $first=0;
@@ -1103,6 +1127,6 @@ __END__
 =head1 AUTHORS
 
  Denis Barbier <barbier@linuxfr.org>
- Martin Quinson <martin.quinson@tuxfamily.org>
+ Martin Quinson (mquinson#debian.org)
 
 =cut
