@@ -40,7 +40,7 @@ Locale::Po4a::Xml is a module to help the translation of XML documents into
 other [human] languages. It can also be used as a base to build modules for
 XML-based documents.
 
-Please note that this module is still under heavy developement, and not
+Please note that this module is still under heavy development, and not
 distributed in official po4a release since we don't feel it to be mature
 enough. If you insist on trying, check the CVS out.
 
@@ -58,6 +58,7 @@ use vars qw(@ISA @EXPORT);
 @EXPORT = qw(new initialize);
 
 use Locale::Po4a::TransTractor;
+use Locale::Po4a::Common;
 use Locale::gettext qw(dgettext);
 
 #It will mantain the path from the root tag to the current one
@@ -114,14 +115,14 @@ These are this module's particular options:
 
 =over 4
 
-=item strip
+=item nostrip
 
-Makes it strip the spaces around the extracted strings. (Typical)
+Prevents it to strip the spaces around the extracted strings.
 
 =item wrap
 
 Canonizes the string to translate, considering that whitespaces are not
-important, and wraps the translated document. This option can be overriden
+important, and wraps the translated document. This option can be overridden
 by custom tag options. See the "tags" option below.
 
 =item caseinsensitive
@@ -175,7 +176,7 @@ sub initialize {
 	my $self = shift;
 	my %options = @_;
 
-	$self->{options}{'strip'}=0;
+	$self->{options}{'nostrip'}=0;
 	$self->{options}{'wrap'}=0;
 	$self->{options}{'caseinsensitive'}=0;
 	$self->{options}{'tagsonly'}=0;
@@ -189,16 +190,18 @@ sub initialize {
 
 	foreach my $opt (keys %options) {
 		if ($options{$opt}) {
-			die sprintf("po4a::xml: ".dgettext ("po4a","Unknown option: %s"), $opt)."\n" unless exists $self->{options}{$opt};
+			die wrap_mod("po4a::xml",
+				dgettext("po4a", "Unknown option: %s"), $opt)
+				unless exists $self->{options}{$opt};
 			$self->{options}{$opt} = $options{$opt};
 		}
 	}
 
-	#It will mantain the list of the translatable tags
+	#It will maintain the list of the translatable tags
 	$self->{tags}=();
-	#It will mantain the list of the translatable attributes
+	#It will maintain the list of the translatable attributes
 	$self->{attributes}=();
-	#It will mantain the list of the inline tags
+	#It will maintain the list of the inline tags
 	$self->{inline}=();
 
 	$self->treat_options;
@@ -284,7 +287,7 @@ sub found_string {
 	} elsif ($options->{'type'} eq "attribute") {
 		$comment = "Attribute '".$options->{'attribute'}."' of: ".$self->get_path;
 	} else {
-		die "po4a::xml: ".sprintf(dgettext("po4a","Internal error: unknown type identifier '%s' at %s."),$options->{'type'},$ref)."\n";
+		die wrap_ref_mod($ref, "po4a::xml", dgettext("po4a", "Internal error: unknown type identifier '%s'."), $options->{'type'});
 	}
 
 	$text = $self->translate($text,$ref,$comment,'wrap'=>$wrap);
@@ -377,7 +380,7 @@ my @tag_types = (
 
 sub tag_extract_comment {
 	my ($self,$remove)=(shift,shift);
-	my ($eof,@tag)=$self->get_string_until('-->',{include=>1,remove=>$remove,unquoted=>1});
+	my ($eof,@tag)=$self->get_string_until('-->',{include=>1,remove=>$remove});
 	return ($eof,@tag);
 }
 
@@ -419,7 +422,7 @@ sub tag_trans_doctype {
 	if (defined $self->{options}{'doctype'} ) {
 		my $doctype = $self->{options}{'doctype'};
 		if ( $tag[0] !~ /\Q$doctype\E/i ) {
-			die "po4a::xml: ".sprintf(dgettext("po4a","Bad document type in %s. '%s' expected."),$tag[1],$doctype)."\n";
+			die wrap_ref_mod($tag[1], "po4a::xml", dgettext("po4a", "Bad document type. '%s' expected."), $doctype);
 		}
 	}
 	my $i = 0;
@@ -468,8 +471,8 @@ sub tag_trans_close {
 	my $name = $self->get_tag_name(@tag);
 
 	my $test = pop @path;
-	if ( $test ne $name ) {
-		die "po4a::xml: ".sprintf(dgettext("po4a","Unexpected closing tag </%s> found in %s. The main document may be wrong."),$name,$tag[1])."\n";
+	if (!defined($test) || $test ne $name ) {
+		die wrap_ref_mod($tag[1], "po4a::xml", dgettext("po4a", "Unexpected closing tag </%s> found. The main document may be wrong."), $name);
 	}
 	return $self->join_lines(@tag);
 }
@@ -580,8 +583,7 @@ sub tag_type {
 	if (!$found) {
 		#It should never enter here, unless you undefine the most
 		#general tags (as <...>)
-		print "po4a::xml: Unknown tag type: ".$line."\n";
-		exit;
+		die "po4a::xml: Unknown tag type: ".$line."\n";
 	} else {
 		return $i;
 	}
@@ -590,9 +592,9 @@ sub tag_type {
 =item extract_tag($$)
 
 This function returns the next tag from the input stream without the beginning
-and end, in an array form, to mantain the references from the input file.  It
+and end, in an array form, to maintain the references from the input file.  It
 has two parameters: the type of the tag (as returned by tag_type) and a
-boolean, that says if it should be removed from the input stream.
+boolean, that indicates if it should be removed from the input stream.
 
 =cut
 
@@ -730,7 +732,7 @@ sub tag_in_list {
 
 =item treat_attributes(@)
 
-This function handles the tags attributes' translation. It receives the tag
+This function handles the translation of the tags' attributes. It receives the tag
 without the beginning / end marks, and then it finds the attributes, and it
 translates the translatable ones (specified by the module option "attributes").
 This returns a plain string with the translated tag.
@@ -795,7 +797,7 @@ sub treat_attributes {
 						if ($self->tag_in_list($self->get_path.$name,@{$self->{attributes}})) {
 							$text .= $self->found_string($value, $ref, { type=>"attribute", attribute=>$name });
 						} else {
-							print sprintf("po4a::xml: ".dgettext ("po4a","Content of attribute %s at %s excluded: %s"),$self->get_path.$name,$ref,$value)."\n"
+							print wrap_ref_mod($ref, "po4a::xml", dgettext("po4a", "Content of attribute %s excluded: %s"), $self->get_path.$name, $value)
 							       if $self->debug();
 							$text .= $self->recode_skipped_text($value);
 						}
@@ -804,7 +806,7 @@ sub treat_attributes {
 				}
 			}
           
-			die sprintf("po4a::xml: ".dgettext ("po4a","Bad attribute syntax at %s"),$ref)."\n"
+			die wrap_ref_mod($ref, "po4a::xml", dgettext ("po4a", "Bad attribute syntax"))
 				unless ($complete);
 		}
 	}
@@ -831,8 +833,8 @@ sub treat_content {
 	}
 
 	# This strips the extracted strings
-	# (only if you specify the 'strip' option)
-	if ($self->{options}{'strip'}) {
+	# (only if you don't specify the 'nostrip' option)
+	if (!$self->{options}{'nostrip'}) {
 		my $clean = 0;
 		# Clean the beginning
 		while (!$clean and $#paragraph > 0) {
@@ -897,7 +899,7 @@ sub treat_content {
 				}));
 		} else {
 			# Inform that this tag isn't translated in debug mode
-			print sprintf("po4a::xml: ".dgettext ("po4a","Content of tag %s at %s excluded: %s"), $self->get_path,$paragraph[1],$self->join_lines(@paragraph))."\n"
+			print wrap_ref_mod($paragraph[1], "po4a::xml", dgettext ("po4a", "Content of tag %s excluded: %s"), $self->get_path, $self->join_lines(@paragraph))
 			       if $self->debug();
 			$self->pushline($self->recode_skipped_text($self->join_lines(@paragraph)));
 		}
@@ -1116,7 +1118,7 @@ L<po4a(7)|po4a.7>, L<Locale::Po4a::TransTractor(3pm)|Locale::Po4a::TransTractor>
 Copyright (c) 2004 by Jordi Vilalta  E<lt>jvprat@wanadoo.esE<gt>
 
 This program is free software; you may redistribute it and/or modify it
-under the terms of GPL (see COPYING file).
+under the terms of GPL (see the COPYING file).
 
 =cut
 
